@@ -20,13 +20,11 @@ import options_storage
 BASE_STAGE = "construction"
 READS_TYPES_USED_IN_CONSTRUCTION = ["paired-end", "single", "hq-mate-pairs"]
 
+
 def prepare_config_spades(filename, cfg, log, additional_contigs_fname, K, stage, saves_dir, last_one, execution_home):
     subst_dict = dict()
 
     subst_dict["K"] = str(K)
-    subst_dict["run_mode"] = "false"
-    if "diploid_mode" in cfg.__dict__:
-        subst_dict["diploid_mode"] = bool_to_str(cfg.diploid_mode)
     subst_dict["dataset"] = process_cfg.process_spaces(cfg.dataset)
     subst_dict["output_base"] = process_cfg.process_spaces(cfg.output_dir)
     subst_dict["tmp_dir"] = process_cfg.process_spaces(cfg.tmp_dir)
@@ -44,11 +42,10 @@ def prepare_config_spades(filename, cfg, log, additional_contigs_fname, K, stage
 #    subst_dict["topology_simplif_enabled"] = bool_to_str(last_one)
     subst_dict["max_threads"] = cfg.max_threads
     subst_dict["max_memory"] = cfg.max_memory
-    subst_dict["correct_mismatches"] = bool_to_str(last_one)
+    if (not last_one):
+        subst_dict["correct_mismatches"] = bool_to_str(False)
     if "resolving_mode" in cfg.__dict__:
         subst_dict["resolving_mode"] = cfg.resolving_mode
-    if "careful" in cfg.__dict__:
-        subst_dict["mismatch_careful"] = bool_to_str(cfg.careful)
     if "pacbio_mode" in cfg.__dict__:
         subst_dict["pacbio_test_on"] = bool_to_str(cfg.pacbio_mode)
         subst_dict["pacbio_reads"] = process_cfg.process_spaces(cfg.pacbio_reads)
@@ -61,6 +58,9 @@ def prepare_config_spades(filename, cfg, log, additional_contigs_fname, K, stage
         else:
             subst_dict["coverage_threshold"] = cfg.cov_cutoff
 
+    #TODO: make something about spades.py and config param substitution 
+    if "bwa_paired" in cfg.__dict__:
+        subst_dict["bwa_enable"] = bool_to_str(True)
     subst_dict["path_to_bwa"] =  os.path.join(execution_home, "bwa-spades")
     process_cfg.substitute_params(filename, subst_dict, log)
 
@@ -96,6 +96,12 @@ def update_k_mers_in_special_cases(cur_k_mers, RL, log, silent=False):
                 support.warning("Default k-mer sizes were set to %s because estimated "
                                 "read length (%d) is equal to or greater than 150" % (str(options_storage.K_MERS_150), RL), log)
             return options_storage.K_MERS_150
+    if RL <= max(cur_k_mers):
+        new_k_mers = [k for k in cur_k_mers if k < RL]
+        if not silent:
+            support.warning("K-mer sizes were set to %s because estimated "
+                            "read length (%d) is less than %d" % (str(new_k_mers), RL, max(cur_k_mers)), log)
+        return new_k_mers
     return cur_k_mers
 
 
@@ -110,13 +116,26 @@ def reveal_original_k_mers(RL):
     original_k_mers = [k for k in original_k_mers if k < RL]
     return original_k_mers
 
+def add_configs(command, configs_dir):
+    #Order matters here!
+    mode_config_mapping = [("single_cell", "mda_mode"), 
+                           ("meta", "meta_mode"), 
+                           ("truseq_mode", "moleculo_mode"),
+                           ("rna", "rna_mode"),
+                           ("plasmid", "plasmid_mode"),
+                           ("careful", "careful_mode"),
+                           ("diploid_mode", "diploid_mode")]
+
+    for (mode, config) in mode_config_mapping:
+        if options_storage.__dict__[mode]:
+            command.append(os.path.join(configs_dir, config + ".info"))
+    
 
 def run_iteration(configs_dir, execution_home, cfg, log, K, prev_K, last_one):
     data_dir = os.path.join(cfg.output_dir, "K%d" % K)
     stage = BASE_STAGE
     saves_dir = os.path.join(data_dir, 'saves')
     dst_configs = os.path.join(data_dir, "configs")
-    cfg_file_name = os.path.join(dst_configs, "config.info")
 
     if options_storage.continue_mode:
         if os.path.isfile(os.path.join(data_dir, "final_contigs.fasta")) and not (options_storage.restart_from and
@@ -135,16 +154,8 @@ def run_iteration(configs_dir, execution_home, cfg, log, K, prev_K, last_one):
             shutil.rmtree(data_dir)
         os.makedirs(data_dir)
 
+        dir_util._path_created = {}  # see http://stackoverflow.com/questions/9160227/dir-util-copy-tree-fails-after-shutil-rmtree
         dir_util.copy_tree(os.path.join(configs_dir, "debruijn"), dst_configs, preserve_times=False)
-        # removing template configs
-        for root, dirs, files in os.walk(dst_configs):
-            for cfg_file in files:
-                cfg_file = os.path.join(root, cfg_file)
-                if cfg_file.endswith('.info.template'):
-                    if os.path.isfile(cfg_file.split('.template')[0]):
-                        os.remove(cfg_file)
-                    else:
-                        os.rename(cfg_file, cfg_file.split('.template')[0])
 
     log.info("\n== Running assembler: " + ("K%d" % K) + "\n")
     if prev_K:
@@ -155,21 +166,29 @@ def run_iteration(configs_dir, execution_home, cfg, log, K, prev_K, last_one):
     else:
         additional_contigs_fname = None
     if "read_buffer_size" in cfg.__dict__:
-        construction_cfg_file_name = os.path.join(dst_configs, "construction.info")
-        process_cfg.substitute_params(construction_cfg_file_name, {"read_buffer_size": cfg.read_buffer_size}, log)
-    prepare_config_spades(cfg_file_name, cfg, log, additional_contigs_fname, K, stage, saves_dir, last_one, execution_home)
+        #FIXME why here???
+        process_cfg.substitute_params(os.path.join(dst_configs, "construction.info"), {"read_buffer_size": cfg.read_buffer_size}, log)
+    if "scaffolding_mode" in cfg.__dict__:
+        #FIXME why here???
+        process_cfg.substitute_params(os.path.join(dst_configs, "pe_params.info"), {"scaffolding_mode": cfg.scaffolding_mode}, log)
 
-    command = [os.path.join(execution_home, "spades"), cfg_file_name]
+    cfg_fn = os.path.join(dst_configs, "config.info")
+    prepare_config_spades(cfg_fn, cfg, log, additional_contigs_fname, K, stage, saves_dir, last_one, execution_home)
+
+    command = [os.path.join(execution_home, "spades"), cfg_fn]
+
+    add_configs(command, dst_configs)
+
+    #print("Calling: " + " ".join(command))
     support.sys_call(command, log)
 
 
-def prepare_config_scaffold_correction(filename, cfg, log, saves_dir, scaffolds_file):
+def prepare_config_scaffold_correction(filename, cfg, log, saves_dir, K):
     subst_dict = dict()
 
-    subst_dict["K"] = str(21)
-    subst_dict["run_mode"] = bool_to_str(False)
+    subst_dict["K"] = str(K)
     subst_dict["dataset"] = process_cfg.process_spaces(cfg.dataset)
-    subst_dict["output_base"] = process_cfg.process_spaces(cfg.output_dir)
+    subst_dict["output_base"] = process_cfg.process_spaces(os.path.join(cfg.output_dir, "SCC"))
     subst_dict["tmp_dir"] = process_cfg.process_spaces(cfg.tmp_dir)
     subst_dict["use_additional_contigs"] = bool_to_str(False)
     subst_dict["main_iteration"] = bool_to_str(False)
@@ -178,15 +197,12 @@ def prepare_config_scaffold_correction(filename, cfg, log, saves_dir, scaffolds_
     subst_dict["developer_mode"] = bool_to_str(cfg.developer_mode)
     subst_dict["max_threads"] = cfg.max_threads
     subst_dict["max_memory"] = cfg.max_memory
-    subst_dict["scaffold_correction_mode"] = bool_to_str(True)
-    subst_dict["scaffolds_file"] = scaffolds_file
 
     #todo
     process_cfg.substitute_params(filename, subst_dict, log)
 
-
-def run_scaffold_correction(configs_dir, execution_home, cfg, log, K):
-    data_dir = os.path.join(cfg.output_dir, "SCC")
+def run_scaffold_correction(configs_dir, execution_home, cfg, log, latest, K):
+    data_dir = os.path.join(cfg.output_dir, "SCC", "K%d" % K)
     saves_dir = os.path.join(data_dir, 'saves')
     dst_configs = os.path.join(data_dir, "configs")
     cfg_file_name = os.path.join(dst_configs, "config.info")
@@ -196,26 +212,18 @@ def run_scaffold_correction(configs_dir, execution_home, cfg, log, K):
     os.makedirs(data_dir)
 
     dir_util.copy_tree(os.path.join(configs_dir, "debruijn"), dst_configs, preserve_times=False)
-    # removing template configs
-    for root, dirs, files in os.walk(dst_configs):
-        for cfg_file in files:
-            cfg_file = os.path.join(root, cfg_file)
-            if cfg_file.endswith('.info.template'):
-                if os.path.isfile(cfg_file.split('.template')[0]):
-                    os.remove(cfg_file)
-                else:
-                    os.rename(cfg_file, cfg_file.split('.template')[0])
 
     log.info("\n== Running scaffold correction \n")
-    latest = os.path.join(cfg.output_dir, "K%d" % K)
     scaffolds_file = os.path.join(latest, "scaffolds.fasta")
     if not os.path.isfile(scaffolds_file):
         support.error("Scaffodls were not found in " + scaffolds_file, log)
     if "read_buffer_size" in cfg.__dict__:
         construction_cfg_file_name = os.path.join(dst_configs, "construction.info")
         process_cfg.substitute_params(construction_cfg_file_name, {"read_buffer_size": cfg.read_buffer_size}, log)
-    prepare_config_scaffold_correction(cfg_file_name, cfg, log, saves_dir, scaffolds_file)
+    process_cfg.substitute_params(os.path.join(dst_configs, "moleculo_mode.info"), {"scaffolds_file": scaffolds_file}, log)
+    prepare_config_scaffold_correction(cfg_file_name, cfg, log, saves_dir, K)
     command = [os.path.join(execution_home, "scaffold_correction"), cfg_file_name]
+    add_configs(command, dst_configs)
     log.info(str(command))
     support.sys_call(command, log)
 
@@ -272,11 +280,15 @@ def run_spades(configs_dir, execution_home, cfg, dataset_data, ext_python_module
             prev_K = K
             RL = get_read_length(cfg.output_dir, K, ext_python_modules_home, log)
             cfg.iterative_K = update_k_mers_in_special_cases(cfg.iterative_K, RL, log)
-            if cfg.iterative_K[1] + 1 > RL:
+            if len(cfg.iterative_K) < 2 or cfg.iterative_K[1] + 1 > RL:
                 if cfg.rr_enable:
-                    support.warning("Second value of iterative K (%d) exceeded estimated read length (%d). "
-                                    "Rerunning for the first value of K (%d) with Repeat Resolving" %
-                                    (cfg.iterative_K[1], RL, cfg.iterative_K[0]), log)
+                    if len(cfg.iterative_K) < 2:
+                        log.info("== Rerunning for the first value of K (%d) with Repeat Resolving" %
+                                 cfg.iterative_K[0])
+                    else:
+                        support.warning("Second value of iterative K (%d) exceeded estimated read length (%d). "
+                                        "Rerunning for the first value of K (%d) with Repeat Resolving" %
+                                        (cfg.iterative_K[1], RL, cfg.iterative_K[0]), log)
                     run_iteration(configs_dir, execution_home, cfg, log, cfg.iterative_K[0], None, True)
                     K = cfg.iterative_K[0]
             else:
@@ -307,8 +319,8 @@ def run_spades(configs_dir, execution_home, cfg, dataset_data, ext_python_module
         else:
             if options_storage.continue_mode:
                 support.continue_from_here(log)
-            run_scaffold_correction(configs_dir, execution_home, cfg, log, K)
-        latest = os.path.join(cfg.output_dir, "SCC")
+            run_scaffold_correction(configs_dir, execution_home, cfg, log, latest, 21)
+        latest = os.path.join(os.path.join(cfg.output_dir, "SCC"), "K21")
         if options_storage.stop_after == 'scc':
             support.finish_here(log)
 
